-------------------------------------------------------------------------------
--
-- Copyright (C) 2012-2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, technology_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE work.unb2b_board_pkg.ALL;
USE technology_lib.technology_pkg.ALL;

ENTITY mms_unb2b_board_system_info IS
  GENERIC (
    g_sim         : BOOLEAN := FALSE;
    g_technology  : NATURAL := c_tech_arria10;
    g_design_name : STRING;
    g_fw_version  : t_unb2b_board_fw_version := c_unb2b_board_fw_version;  -- firmware version x.y
    g_stamp_date  : NATURAL := 0;
    g_stamp_time  : NATURAL := 0;
    g_revision_id : STRING  := "";
    g_design_note : STRING  := "";
    g_rom_version : NATURAL := 1;
    g_aux         : t_c_unb2b_board_aux := c_unb2b_board_aux               -- aux contains the hardware version
  );
  PORT (
    mm_rst          : IN    STD_LOGIC;
    mm_clk          : IN    STD_LOGIC;

    -- MM registers
    reg_mosi        : IN    t_mem_mosi := c_mem_mosi_rst;
    reg_miso        : OUT   t_mem_miso;

    rom_mosi        : IN    t_mem_mosi := c_mem_mosi_rst;
    rom_miso        : OUT   t_mem_miso;

    hw_version      : IN  STD_LOGIC_VECTOR(g_aux.version_w-1 DOWNTO 0);
    id              : IN  STD_LOGIC_VECTOR(g_aux.id_w-1 DOWNTO 0);

    chip_id         : OUT STD_LOGIC_VECTOR(c_unb2b_board_nof_chip_w-1 DOWNTO 0);
    bck_id          : OUT STD_LOGIC_VECTOR(c_unb2b_board_nof_uniboard_w-1 DOWNTO 0);

    -- Info output still supported for older designs
    info            : OUT STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0)
    );
END mms_unb2b_board_system_info;


ARCHITECTURE str OF mms_unb2b_board_system_info IS

  -- Provide different prefixes (absolute and relative) for the same path. ModelSim understands $UNB, Quartus does not.
  -- Required because the work paths of ModelSim and Quartus are different.
  CONSTANT c_quartus_path_prefix  : STRING := "";
  CONSTANT c_modelsim_path_prefix : STRING := "$UNB/Firmware/designs/" & g_design_name & "/build/synth/quartus/";
  CONSTANT c_path_prefix          : STRING := sel_a_b(g_sim, c_modelsim_path_prefix, c_quartus_path_prefix);

-- No longer supporting MIF files in sim as non-$UNB (e.g. $AARTFAAC) designs will cause path error.
--  CONSTANT c_mif_name    : STRING := sel_a_b((g_design_name="UNUSED"), g_design_name, c_path_prefix & g_design_name & ".mif");
  CONSTANT c_mif_name    : STRING :=  sel_a_b(g_sim, "UNUSED", sel_a_b((g_design_name="UNUSED"), g_design_name, c_path_prefix & g_design_name & ".mif"));

  CONSTANT c_rom_addr_w  : NATURAL := 13; -- 2^13 = 8192 addresses * 32 bits = 32 kiB

  CONSTANT c_mm_rom      : t_c_mem := (latency  => 1,
                                      adr_w    => c_rom_addr_w,
                                      dat_w    => c_word_w,
                                      nof_dat  => 2**c_rom_addr_w,  -- = 2**adr_w
                                      init_sl  => '0');
 
  SIGNAL i_info          : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);

BEGIN

 info <= i_info;
 
  u_unb2b_board_system_info: ENTITY work.unb2b_board_system_info
  GENERIC MAP (
    g_sim        => g_sim,
    g_fw_version => g_fw_version,
    g_rom_version => g_rom_version,
    g_technology  => g_technology
  )            
  PORT MAP (        
    clk        => mm_clk, 
    hw_version => hw_version,
    id         => id,
    info       => i_info,
    chip_id    => chip_id,
    bck_id     => bck_id
   );

  u_unb2b_board_system_info_reg: ENTITY work.unb2b_board_system_info_reg                   
  GENERIC MAP (
    g_design_name => g_design_name,
    g_stamp_date  => g_stamp_date,
    g_stamp_time  => g_stamp_time,
    g_revision_id => g_revision_id,
    g_design_note => g_design_note
  )    
  PORT MAP (       
    mm_rst    => mm_rst, 
    mm_clk    => mm_clk,

    sla_in    => reg_mosi,
    sla_out   => reg_miso,

    info      => i_info
  );

  u_common_rom : ENTITY common_lib.common_rom
  GENERIC MAP (
    g_technology => g_technology,
    g_ram       => c_mm_rom,
    g_init_file => c_mif_name
  )
  PORT MAP (
    rst     => mm_rst,
    clk     => mm_clk,
    rd_en   => rom_mosi.rd,
    rd_adr  => rom_mosi.address(c_mm_rom.adr_w-1 DOWNTO 0),
    rd_dat  => rom_miso.rddata(c_mm_rom.dat_w-1 DOWNTO 0),
    rd_val  => rom_miso.rdval
  );
  
END str;

