-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Test bench for mms_unb2b_board_sens
--
-- Features:
-- . Verify that the UniBoard sensors are read.
--
-- Usage:
-- . > as 10
-- . > run -all

ENTITY tb_mms_unb2b_board_sens IS
END tb_mms_unb2b_board_sens;

LIBRARY IEEE, common_lib, i2c_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;
USE common_lib.tb_common_mem_pkg.ALL;
USE work.unb2b_board_pkg.ALL;

ARCHITECTURE tb OF tb_mms_unb2b_board_sens IS

  CONSTANT c_sim              : BOOLEAN := TRUE;  --FALSE;
  CONSTANT c_repeat           : NATURAL := 2;
  CONSTANT c_clk_freq         : NATURAL := 100*10**6;
  CONSTANT c_clk_period       : TIME    := (10**9/c_clk_freq) * 1 ns;  
  CONSTANT c_rst_period       : TIME    := 4 * c_clk_period;  
  
  -- Model I2C sensor slaves as on the UniBoard
  CONSTANT c_temp_high           : NATURAL := 85;
  CONSTANT c_fpga_temp_address   : STD_LOGIC_VECTOR(6 DOWNTO 0) := "0011000";  -- MAX1618 address LOW LOW
  CONSTANT c_fpga_temp           : INTEGER := 60;
  CONSTANT c_eth_temp_address    : STD_LOGIC_VECTOR(6 DOWNTO 0) := "0101001";  -- MAX1618 address MID LOW
  CONSTANT c_eth_temp            : INTEGER := 40;
  CONSTANT c_hot_swap_address    : STD_LOGIC_VECTOR(6 DOWNTO 0) := "1000100";  -- LTC4260 address L L L
  CONSTANT c_hot_swap_R_sense    : REAL := 0.01;                               -- = 10 mOhm on UniBoard
  
  CONSTANT c_uniboard_current    : REAL := 5.0;   -- = assume 5.0 A on UniBoard  --> hot swap = 5010 mAmpere (167)
  CONSTANT c_uniboard_supply     : REAL := 48.0;  -- = assume 48.0 V on UniBoard --> hot swap = 48000 mVolt (120)
  CONSTANT c_uniboard_adin       : REAL := -1.0;  -- = NC on UniBoard
  
  CONSTANT c_sens_nof_result  : NATURAL := 4 + 1;
  CONSTANT c_sens_expected    : t_natural_arr(0 TO c_sens_nof_result-1) := (60, 40, 167, 120, 0);  -- 4 bytes as read by c_SEQ in unb2b_board_sens_ctrl + sens_err
  
  SIGNAL tb_end          : STD_LOGIC := '0';
  SIGNAL clk             : STD_LOGIC := '0';
  SIGNAL rst             : STD_LOGIC := '1';
  SIGNAL start           : STD_LOGIC;
 
  SIGNAL reg_mosi        : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_miso        : t_mem_miso;
  
  SIGNAL sens_val        : STD_LOGIC;
  SIGNAL sens_dat        : STD_LOGIC_VECTOR(c_byte_w-1 DOWNTO 0);
  
  SIGNAL scl_stretch     : STD_LOGIC := 'Z';
  SIGNAL scl             : STD_LOGIC;
  SIGNAL sda             : STD_LOGIC;  
  
BEGIN

  rst <= '0' AFTER 4*c_clk_period;
  clk <= (NOT clk) OR tb_end AFTER c_clk_period/2;
  
  -- I2C bus
  scl <= 'H';   -- model I2C pull up
  sda <= 'H';   -- model I2C pull up, use '0' and '1' to verify sens_err

  scl <= scl_stretch;

  sens_clk_stretch : PROCESS (scl)
  BEGIN
    IF falling_edge(scl) THEN
      scl_stretch <= '0', 'Z' AFTER 50 ns;   -- < 10 ns to effectively disable stretching, >= 50 ns to enable it
    END IF;
  END PROCESS;
  
  p_mm_reg_stimuli : PROCESS
    VARIABLE v_bsn : NATURAL;
    VARIABLE vI    : NATURAL;
    VARIABLE vJ    : NATURAL;
  BEGIN
    start     <= '0';
    reg_mosi  <= c_mem_mosi_rst;
    
    proc_common_wait_until_low(clk, rst);
    proc_common_wait_some_cycles(clk, 10);
    
    FOR I IN 0 TO c_repeat-1 LOOP
      -- start I2C access
      start <= '1';
      proc_common_wait_some_cycles(clk, 1);
      start <= '0';
      
      -- wait for I2C access to have finished
      proc_common_wait_some_cycles(clk, sel_a_b(c_sim, 5000, 500000));
      
      -- read I2C result data
      FOR I IN 0 TO c_sens_nof_result-1 LOOP
        proc_mem_mm_bus_rd(I, clk, reg_miso, reg_mosi);  -- read sens_data
      END LOOP;
      
      proc_common_wait_some_cycles(clk, 1000);
    END LOOP;
    
    proc_common_wait_some_cycles(clk, 100);
    tb_end <= '1';
    WAIT;
  END PROCESS;
  
  sens_val <= reg_miso.rdval;
  sens_dat <= reg_miso.rddata(c_byte_w-1 DOWNTO 0);
  
  -- Verify sensor data
  p_verify : PROCESS
  BEGIN
    WAIT UNTIL rising_edge(clk);  -- Added this line to avoid warning: (vcom-1090) Possible infinite loop: Process contains no WAIT statement.
    
    proc_common_wait_until_high(clk, sens_val);
    ASSERT TO_UINT(sens_dat)=c_sens_expected(0) REPORT "Wrong FPGA temperature value" SEVERITY ERROR;
    proc_common_wait_some_cycles(clk, 1);
    ASSERT TO_UINT(sens_dat)=c_sens_expected(1) REPORT "Wrong ETH temperature value" SEVERITY ERROR;
    proc_common_wait_some_cycles(clk, 1);
    ASSERT TO_UINT(sens_dat)=c_sens_expected(2) REPORT "Wrong hot swap V sense value" SEVERITY ERROR;
    proc_common_wait_some_cycles(clk, 1);
    ASSERT TO_UINT(sens_dat)=c_sens_expected(3) REPORT "Wrong hot swap V source value" SEVERITY ERROR;
    proc_common_wait_some_cycles(clk, 1);
    ASSERT TO_UINT(sens_dat)=c_sens_expected(4) REPORT "An I2C error occurred" SEVERITY ERROR;
    
  END PROCESS;
  

  -- I2C sensors master
  u_mms_unb2b_board_sens : ENTITY work.mms_unb2b_board_sens
  GENERIC MAP (
    g_sim             => c_sim,
    g_i2c_peripheral  => c_i2c_peripheral_sens,
    g_sens_nof_result => 40,
    g_clk_freq        => c_clk_freq,
    g_temp_high       => c_temp_high,
    g_comma_w         => 13
  )
  PORT MAP (
    -- Clocks and reset
    mm_rst    => rst,
    mm_clk    => clk,
    mm_start  => start,
    
    -- Memory-mapped clock domain
    reg_mosi  => reg_mosi,
    reg_miso  => reg_miso,
    
    -- i2c bus
    scl       => scl,
    sda       => sda
  );
  
  -- I2C slaves that are available for each FPGA
  u_fpga_temp : ENTITY i2c_lib.dev_max1618
  GENERIC MAP (
    g_address => c_fpga_temp_address
  )
  PORT MAP (
    scl  => scl,
    sda  => sda,
    temp => c_fpga_temp
  );

  -- I2C slaves that are available only via FPGA node 3
  u_eth_temp : ENTITY i2c_lib.dev_max1618
  GENERIC MAP (
    g_address => c_eth_temp_address
  )
  PORT MAP (
    scl  => scl,
    sda  => sda,
    temp => c_eth_temp
  );
  
  u_power : ENTITY i2c_lib.dev_ltc4260
  GENERIC MAP (
    g_address => c_hot_swap_address,
    g_R_sense => c_hot_swap_R_sense
  )
  PORT MAP (
    scl               => scl,
    sda               => sda,
    ana_current_sense => c_uniboard_current,
    ana_volt_source   => c_uniboard_supply,
    ana_volt_adin     => c_uniboard_adin
  );
    
END tb;

